==> flashgeotext-0.4.0.tar.gz: flashgeotext-0.4.0/flashgeotext/geotext.py <==
from pydantic import BaseModel

from flashgeotext.lookup import LookupDataPool, MissingLookupDataError


class GeoTextConfiguration(BaseModel):
    """GeoText configuration

    Args:
    use_demo_data (bool): load demo data or not, default True
    case_sensitive (bool): case sensitive lookup, default True
    """

    use_demo_data: bool = True
    case_sensitive: bool = True


CONFIG = GeoTextConfiguration()


class GeoText(LookupDataPool):
    """Extract LookupData from input text

    GeoText inherits from LookupDataPool. It iterates through
    a LookupDataPool and interfaces flashtext.KeywordProcessor
    to extract keywords from an input text. It parses the result of
    flashtext.KeywordProcessor.extract_keywords() by counting the
    occurances of a LookupData point and optionally lists the
    span info.

    Example:
    ```python
        from flashgeotext.geotext import GeoText

        geotext = GeoText()

        input_text = '''Shanghai. The Chinese Ministry of Finance in Shanghai said that China plans
                        to cut tariffs on $75 billion worth of goods that the country
                        imports from the US. Washington welcomes the decision.'''

        geotext.extract(input_text=input_text)
        >> {
            'cities': {
                'Shanghai': {
                    'count': 2,
                    'span_info': [(0, 8), (45, 53)]
                    'found_as': ['Shanghai', 'Shanghai']
                    },
                'Washington, D.C.': {
                    'count': 1,
                    'span_info': [(175, 185)]
                    'found_as': ['Washington']
                    }
                },
            'countries': {
                'China': {
                    'count': 1,
                    'span_info': [(64, 69)]
                    'found_as': ['China']
                    },
                'United States': {
                    'count': 1,
                    'span_info': [(171, 173)]
                    'found_as': ['US']
                    }
                }
            }

    ```

    """

    def __init__(self, config: GeoTextConfiguration = CONFIG) -> None:
        """instantiate an empty LookupDataPool, optionally/by default with demo data

        Args:
            config: GeoTextConfiguration = { use_demo_data: True, case_sensitive: True }.
        """
        self.pool: dict = {}

        if config.use_demo_data:
            self._add_demo_data(case_sensitive=config.case_sensitive)

    def extract(self, input_text: str, span_info: bool = True) -> dict:
        """Extract LookupData from an input_text

        Args:
            input_text (str): String to extract LookupData from.
            span_info (bool): Optionally, return span_info. Defaults to True.

        Returns:
            extract_ouput (dict): dictionary of extracted LookupData entities with count
                                  and optionally with listed span info.

        """
        if not self.pool:
            raise MissingLookupDataError(
                "Empty LookupDataPool. use .add(LookupData) to add data."
            )

        output: dict = {}

        for lookup in self.pool.keys():
            extract = self.pool[lookup].extract_keywords(
                input_text, span_info=span_info
            )
            output[lookup] = self._parse_extract(extract, input_text)

        return output

    def _parse_extract(self, extract_data: list, input_text: str) -> dict:
        """Parse flashtext.KeywordProcessor.extract_keywords() output to count occurances

        Parse flashtext.KeywordProcessor.extract_keywords() output to count occurances,
        and optionally span_info.

        Args:
            extract_data (list): flashtext.KeywordProcessor.extract_keywords() return value
            input_text (str): input text

        Returns:
            parsed_extract (dict): parsed extract_data to include count, optionally span_info
        """
        parsed_extract: dict = {}

        for entry in extract_data:
            keyword = entry[0]
            span_start = entry[1]
            span_end = entry[2]

            if keyword not in parsed_extract:
                parsed_extract[keyword] = {
                    "count": 1,
                    "span_info": [(span_start, span_end)],
                    "found_as": [input_text[span_start:span_end]],
                }
            else:
                parsed_extract[keyword]["count"] = parsed_extract[keyword]["count"] + 1
                parsed_extract[keyword]["span_info"] = parsed_extract[keyword][
                    "span_info"
                ] + [(span_start, span_end)]
                parsed_extract[keyword]["found_as"] = parsed_extract[keyword][
                    "found_as"
                ] + [input_text[span_start:span_end]]

        return parsed_extract

==> flashgeotext-0.4.2.tar.gz: flashgeotext-0.4.2/flashgeotext/geotext.py <==
from pydantic import BaseModel

from flashgeotext.lookup import LookupDataPool, MissingLookupDataError


class GeoTextConfiguration(BaseModel):
    """GeoText configuration

    Args:
    use_demo_data (bool): load demo data or not, default True
    case_sensitive (bool): case sensitive lookup, default True
    """

    use_demo_data: bool = True
    case_sensitive: bool = True


CONFIG = GeoTextConfiguration()


class GeoText(LookupDataPool):
    """Extract LookupData from input text

    GeoText inherits from LookupDataPool. It iterates through
    a LookupDataPool and interfaces flashtext.KeywordProcessor
    to extract keywords from an input text. It parses the result of
    flashtext.KeywordProcessor.extract_keywords() by counting the
    occurances of a LookupData point and optionally lists the
    span info.

    Example:
    ```python
        from flashgeotext.geotext import GeoText

        geotext = GeoText()

        input_text = '''Shanghai. The Chinese Ministry of Finance in Shanghai said that China plans
                        to cut tariffs on $75 billion worth of goods that the country
                        imports from the US. Washington welcomes the decision.'''

        geotext.extract(input_text=input_text)
        >> {
            'cities': {
                'Shanghai': {
                    'count': 2,
                    'span_info': [(0, 8), (45, 53)]
                    'found_as': ['Shanghai', 'Shanghai']
                    },
                'Washington, D.C.': {
                    'count': 1,
                    'span_info': [(175, 185)]
                    'found_as': ['Washington']
                    }
                },
            'countries': {
                'China': {
                    'count': 1,
                    'span_info': [(64, 69)]
                    'found_as': ['China']
                    },
                'United States': {
                    'count': 1,
                    'span_info': [(171, 173)]
                    'found_as': ['US']
                    }
                }
            }

    ```

    """

    def __init__(self, config: GeoTextConfiguration = CONFIG) -> None:
        """instantiate an empty LookupDataPool, optionally/by default with demo data

        Args:
            config: GeoTextConfiguration = { use_demo_data: True, case_sensitive: True }.
        """
        self.pool: dict = {}

        if config.use_demo_data:
            self._add_demo_data(case_sensitive=config.case_sensitive)

    def extract(self, input_text: str, span_info: bool = True) -> dict:
        """Extract LookupData from an input_text

        Args:
            input_text (str): String to extract LookupData from.
            span_info (bool): Optionally, return span_info. Defaults to True.

        Returns:
            extract_ouput (dict): dictionary of extracted LookupData entities with count
                                  and optionally with listed span info.

        """
        if not self.pool:
            raise MissingLookupDataError(
                "Empty LookupDataPool. use .add(LookupData) to add data."
            )

        output: dict = {}

        for lookup in self.pool.keys():
            extract = self.pool[lookup].extract_keywords(
                input_text, span_info=span_info
            )
            output[lookup] = self.__parse_extract(extract, input_text, span_info)

        return output

    def __parse_extract(
        self, extract_data: list, input_text: str, span_info: bool
    ) -> dict:
        """Parse flashtext.KeywordProcessor.extract_keywords() output to count occurances

        Parse flashtext.KeywordProcessor.extract_keywords() output to count occurances,
        and optionally span_info.

        Args:
            extract_data (list): flashtext.KeywordProcessor.extract_keywords() return value
            input_text (str): input text

        Returns:
            parsed_extract (dict): parsed extract_data to include count, optionally span_info
        """

        if span_info:
            parsed_extract = {}
            for entry in extract_data:
                keyword = entry[0]
                span_start = entry[1]
                span_end = entry[2]

                if keyword not in parsed_extract:
                    parsed_extract[keyword] = {
                        "count": 1,
                        "span_info": [(span_start, span_end)],
                        "found_as": [input_text[span_start:span_end]],
                    }
                else:
                    parsed_extract[keyword]["count"] = (
                        parsed_extract[keyword]["count"] + 1
                    )
                    parsed_extract[keyword]["span_info"] = parsed_extract[keyword][
                        "span_info"
                    ] + [(span_start, span_end)]
                    parsed_extract[keyword]["found_as"] = parsed_extract[keyword][
                        "found_as"
                    ] + [input_text[span_start:span_end]]

            return parsed_extract

        parsed_extract = {}

        for entry in extract_data:
            keyword = entry
            if keyword not in parsed_extract:
                parsed_extract[keyword] = {"count": 1}
            else:
                parsed_extract[keyword]["count"] = parsed_extract[keyword]["count"] + 1

        return parsed_extract
